class Technology(object):

    def __init__(self, name=None, category=None, version=None) -> None:
        # Fields
        self.name = name
        self.category = category
        self.version = version

    def __repr__(self):
        return "<Technology name: %r, category: %r, version: %r>" % (
            self.name,
            self.category,
            self.version,
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.http_transaction import HttpTransaction


class HttpTransactionSchema(Schema):
    url = fields.String(allow_none=True)
    ip = fields.String(allow_none=True)
    method = fields.String(allow_none=True)
    mimetypes = fields.String(allow_none=True)
    content_type = fields.String(allow_none=True)
    status_code = fields.Int(allow_none=True)
    size = fields.Float(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return HttpTransaction(**data)


http_transactions_schema = HttpTransactionSchema(many=True)



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.console_message import ConsoleMessage


class ConsoleMessageSchema(Schema):
    text = fields.String(allow_none=True)
    type = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return ConsoleMessage(**data)


console_messages_schema = ConsoleMessageSchema(many=True)



class FastCheck(object):

    def __init__(self, url=None, malicious_score=None, status=None, scan_id=None):
        """
        Init fast check
        :param url: Submitted url
        :param malicious_score: Malicious score [0-1]
        :param status: Status ["SAFE","MALICIOUS","UNRATED"]
        :param scan_id: matched Scan ID
        """
        self.url = url
        self.malicious_score = malicious_score
        self.status = status
        self.scan_id = scan_id

    def __repr__(self):
        return "<FastCheck url: %r malicious_score: %r status: %r>" % (
            self.url,
            self.malicious_score,
            self.status,
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.scan import Scan


class ScanSchema(Schema):
    id = fields.String(allow_none=True)
    submitted_url = fields.String(allow_none=True)
    target_url = fields.String(allow_none=True)
    protocol = fields.String(allow_none=True)
    device = fields.String(allow_none=True)
    user_agent = fields.String(allow_none=True)
    http_referer = fields.String(allow_none=True)
    nsfw = fields.Boolean(allow_none=True)
    scanned_from = fields.String(allow_none=True)
    origin = fields.String(allow_none=True)
    width = fields.Integer(allow_none=True)
    height = fields.Integer(allow_none=True)
    domain = fields.String(allow_none=True)
    private_scan = fields.Boolean(allow_none=True)
    status = fields.String(allow_none=True)
    submitted_date = fields.DateTime(allow_none=True)
    waiting_time = fields.Integer(allow_none=True)
    submitter_tags = fields.List(fields.String, required=False, allow_none=True)
    http_referer = fields.String(allow_none=True)
    error_code = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Scan(**data)


scan_schema = ScanSchema()
scans_schema = ScanSchema(many=True)



# Marshmallow
from marshmallow import Schema, fields, post_load

# Model
from urldna.models.user_agent import UserAgent


class UserAgentSchema(Schema):
    device = fields.String(allow_none=True)
    browser = fields.String(allow_none=True)
    user_agent = fields.String(allow_none=True)
    share = fields.Float(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return UserAgent(**data)


user_agent_schema = UserAgentSchema()
user_agents_schema = UserAgentSchema(many=True)



class Certificate(object):

    def __init__(
        self,
        authority_info_access=None,
        authority_key_identifier=None,
        basic_constraints=None,
        certificate_policies=None,
        ct_precert_scts=None,
        extended_key_usage=None,
        issuer=None,
        key_usage=None,
        not_after=None,
        not_before=None,
        serial_number=None,
        subject=None,
        subject_key_identifier=None,
        version=None,
    ) -> None:
        # Fields
        self.authority_info_access = authority_info_access
        self.authority_key_identifier = authority_key_identifier
        self.basic_constraints = basic_constraints
        self.certificate_policies = certificate_policies
        self.ct_precert_scts = ct_precert_scts
        self.extended_key_usage = extended_key_usage
        self.issuer = issuer
        self.key_usage = key_usage
        self.not_after = not_after
        self.not_before = not_before
        self.serial_number = serial_number
        self.subject = subject
        self.subject_key_identifier = subject_key_identifier
        self.version = version

    def __repr__(self):
        return "<Certificate issuer: %r, subject: %r, serial_number: %r>" % (
            self.issuer,
            self.subject,
            self.serial_number,
        )



class RedirectChain(object):

    def __init__(self, redirect_url=None, status=None, elapsed_time=None) -> None:
        # Fields
        self.redirect_url = redirect_url
        self.status = status
        self.elapsed_time = elapsed_time

    def __repr__(self):
        return "<RedirectChain redirect_url: %r, status: %r, elapsed_time: %r>" % (
            self.redirect_url[:50],
            self.status,
            self.elapsed_time,
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.screenshot import Screenshot


class ScreenshotSchema(Schema):
    blob_uri = fields.String(allow_none=True)
    name = fields.String(allow_none=True)
    phash = fields.String(allow_none=True)
    width = fields.Integer(allow_none=True)
    height = fields.Integer(allow_none=True)
    mimetype = fields.String(allow_none=True)
    format = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Screenshot(**data)


screenshot_schema = ScreenshotSchema()



class Page(object):

    def __init__(
        self,
        text=None,
        title=None,
        blob_uri=None,
        meta_tags=None,
        headers=None,
        ssdeep=None,
        outgoing_links=None,
    ) -> None:
        # Fields
        self.text = text
        self.title = title
        self.blob_uri = blob_uri
        self.meta_tags = meta_tags
        self.headers = headers
        self.ssdeep = ssdeep
        self.outgoing_links = outgoing_links

    def __repr__(self):
        return "<Page title: %r, text: %r>" % (self.title, self.text[:50])



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.fast_check import FastCheck


class FastCheckSchema(Schema):
    url = fields.String(allow_none=True)
    malicious_score = fields.Float(allow_none=True)
    status = fields.String(allow_none=True)
    scan_id = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return FastCheck(**data)


fast_check_schema = FastCheckSchema()



class SearchResult(object):

    def __init__(
        self,
        search_id=None,
        query=None,
        elapsed_time=None,
        fetched_result=None,
        search_date=None,
        scan_results=[],
    ):
        """
        Init Search result
        :param search_id: Search ID
        :param query: Query
        :param elapsed_time: Elapsed Time
        :param fetched_result: Fetched Result
        :param search_date: Search Date
        :param user_id: User ID
        :param scan_results: list of ScanResult
        """
        self.search_id = search_id
        self.query = query
        self.elapsed_time = elapsed_time
        self.fetched_result = fetched_result
        self.search_date = search_date
        self.scan_results = scan_results

    def __repr__(self):
        return "<SearchResult query: %r fetched_result: %r>" % (
            str(self.query),
            str(self.fetched_result),
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.redirect_chain import RedirectChain


class RedirectChainSchema(Schema):
    redirect_url = fields.String(allow_none=True)
    status = fields.String(allow_none=True)
    elapsed_time = fields.Float(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return RedirectChain(**data)


redirect_chains_schema = RedirectChainSchema(many=True)



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.cookie import Cookie


class CookieSchema(Schema):
    domain = fields.String(allow_none=True)
    expiry = fields.DateTime(allow_none=True)
    http_only = fields.Boolean(allow_none=True)
    name = fields.String(allow_none=True)
    path = fields.String(allow_none=True)
    secure = fields.Boolean(allow_none=True)
    value = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Cookie(**data)


cookies_schema = CookieSchema(many=True)



class ScanWhois(object):

    def __init__(
        self,
        domain_name=None,
        registrar=None,
        creation_date=None,
        expiration_date=None,
        updated_date=None,
        name=None,
        org=None,
        address=None,
        registrant_postal_code=None,
        country=None,
        state=None,
    ) -> None:
        # Fields
        self.domain_name = domain_name
        self.registrar = registrar
        self.creation_date = creation_date
        self.expiration_date = expiration_date
        self.updated_date = updated_date
        self.name = name
        self.org = org
        self.address = address
        self.registrant_postal_code = registrant_postal_code
        self.country = country
        self.state = state

    def __repr__(self):
        return "<ScanWhois domain_name: %r, registrar: %r>" % (
            self.domain_name,
            self.registrar,
        )



class ConsoleMessage(object):

    def __init__(self, text=None, type=None) -> None:
        # Fields
        self.text = text
        self.type = type

    def __repr__(self):
        return "<ConsoleMessage text: %r, type: %r>" % (self.text[:50], self.type)



class Malicious(object):

    def __init__(self, malicious=None, threat=None) -> None:

        # Fields
        self.malicious = malicious
        self.threat = threat

    def __repr__(self):
        return "<Malicious threat: %r, malicious: %r>" % (self.threat, self.malicious)



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.certificate import Certificate


class CertificateSchema(Schema):
    authority_info_access = fields.String(allow_none=True)
    authority_key_identifier = fields.String(allow_none=True)
    basic_constraints = fields.String(allow_none=True)
    certificate_policies = fields.String(allow_none=True)
    ct_precert_scts = fields.String(allow_none=True)
    extended_key_usage = fields.String(allow_none=True)
    issuer = fields.String(allow_none=True)
    key_usage = fields.String(allow_none=True)
    not_after = fields.DateTime(allow_none=True)
    not_before = fields.DateTime(allow_none=True)
    serial_number = fields.String(allow_none=True)
    subject = fields.String(allow_none=True)
    subject_key_identifier = fields.String(allow_none=True)
    version = fields.Integer(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Certificate(**data)


certificate_schema = CertificateSchema()



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.technology import Technology


class TechnologySchema(Schema):
    name = fields.String(allow_none=True)
    category = fields.String(allow_none=True)
    version = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Technology(**data)


technologies_schema = TechnologySchema(many=True)



class IpAddress(object):

    def __init__(
        self,
        ip=None,
        asn=None,
        city=None,
        country=None,
        country_code=None,
        isp=None,
        latitude=None,
        longitude=None,
        org=None,
        region=None,
        timezone_gmt=None,
        type=None,
    ) -> None:
        # Fields
        self.ip = ip
        self.asn = asn
        self.city = city
        self.country = country
        self.country_code = country_code
        self.isp = isp
        self.latitude = latitude
        self.longitude = longitude
        self.org = org
        self.region = region
        self.timezone_gmt = timezone_gmt
        self.type = type

    def __repr__(self):
        return "<IpAddress ip: %r, asn: %r, country: %r>" % (
            self.ip,
            self.asn,
            self.country,
        )



class Cookie(object):

    def __init__(
        self,
        domain=None,
        expiry=None,
        secure=None,
        http_only=None,
        name=None,
        path=None,
        value=None,
    ) -> None:
        # Fields
        self.domain = domain
        self.expiry = expiry
        self.secure = secure
        self.http_only = http_only
        self.name = name
        self.path = path
        self.value = value

    def __repr__(self):
        return "<Cookie name: %r, domain: %r, value: %r>" % (
            self.name,
            self.domain,
            self.value[:50],
        )



class Screenshot(object):

    def __init__(
        self,
        blob_uri=None,
        phash=None,
        width=None,
        height=None,
        mimetype=None,
        format=None,
    ) -> None:

        # Fields
        self.blob_uri = blob_uri
        self.phash = phash
        self.width = width
        self.height = height
        self.mimetype = mimetype
        self.format = format

    def __repr__(self):
        return "<Screenshot phash: %r, blob_uri: %r>" % (self.phash, self.blob_uri)



class ScanResult(object):

    def __init__(
        self,
        scan=None,
        ip_address=None,
        certificate=None,
        favicon=None,
        screenshot=None,
        page=None,
        dom=None,
        cookies=None,
        technologies=None,
        scan_feedback=None,
        http_transactions=None,
        console_messages=None,
        malicious=None,
        scan_whois=None,
        redirect_chains=None,
    ):
        """
        Init Full Scan result
        :param scan: Scan
        :param ip_address: IpAddress
        :param certificate: Certificate
        :param favicon: Favicon
        :param screenshot: Screenshot
        :param dom: Page DOM
        :param cookies: List of Cookie
        :param scan_feedback: Scan Feedback
        :param technologies: List of Technology
        :param http_transactions: List of HttpTransaction
        :param console_messages: List of ConsoleMessage
        :param malicious: Malicious
        :param scan_whois: ScanWhois
        :param redirect_chains: List of RedirectChain
        """
        self.scan = scan
        self.ip_address = ip_address
        self.certificate = certificate
        self.favicon = favicon
        self.screenshot = screenshot
        self.page = page
        self.dom = dom
        self.cookies = cookies
        self.technologies = technologies
        self.scan_feedback = scan_feedback
        self.http_transactions = http_transactions
        self.console_messages = console_messages
        self.malicious = malicious
        self.scan_whois = scan_whois
        self.redirect_chains = redirect_chains

    def __repr__(self):
        return "<ScanResult scan: %r >" % self.scan



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.malicious import Malicious


class MaliciousSchema(Schema):
    malicious = fields.Boolean(allow_none=True)
    threat = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Malicious(**data)


malicious_schema = MaliciousSchema()



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.scan_whois import ScanWhois


class ScanWhoisSchema(Schema):
    domain_name = fields.String(allow_none=True)
    registrar = fields.String(allow_none=True)
    creation_date = fields.DateTime(allow_none=True)
    expiration_date = fields.DateTime(allow_none=True)
    updated_date = fields.DateTime(allow_none=True)
    name = fields.String(allow_none=True)
    org = fields.String(allow_none=True)
    address = fields.String(allow_none=True)
    registrant_postal_code = fields.String(allow_none=True)
    country = fields.String(allow_none=True)
    state = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return ScanWhois(**data)


scan_whois_schema = ScanWhoisSchema()



import time
import requests

# Exception
from urldna.exceptions.api_exception import ApiException

# Schemas
from urldna.schemas.scan_schema import scan_schema, scans_schema
from urldna.schemas.viewport_schema import viewports_schema
from urldna.schemas.scan_result_schema import scan_result_schema
from urldna.schemas.user_agent_schema import user_agents_schema
from urldna.schemas.fast_check_schema import fast_check_schema
from urldna.schemas.scan_feedback_schema import scan_feedback_schema


class UrlDNA:

    # URL
    ENDPOINT_URL = "https://api.urldna.io/v1"

    def __init__(self, api_key):
        """
        Initialize urlDNA client with API KEY, register to get a FREE API key.
        :param api_key: API key, see https://urldna.io/profile
        """
        self.api_key = api_key
        self.headers = {
            "Authorization": self.api_key,
            "Content-Type": "application/json",
        }

    def get_scan(self, scan_id):
        """
        Get Scan by Scan ID
        :param scan_id: Scan ID
        :return: Scan object
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/scan/" + str(scan_id)

        # Get response
        response = requests.get(api_url, headers=self.headers)

        if response.status_code == 200:
            return scan_result_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def search(self, query, page=1):
        """
        Search by query text
        :param query: Domain or CQL syntax
        :param page: Pagination (only for PREMIUM users)
        :return: Search results
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/search"
        # payload
        payload = {"query": query}
        # params
        params = {"page": page}

        # Get response
        response = requests.post(
            api_url, headers=self.headers, json=payload, params=params
        )

        if response.status_code == 200:
            return scans_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def viewports(self):
        """
        Get list of all available viewports
        :return: List of viewports (device, height, width)
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/viewports"

        # Get response
        response = requests.get(api_url, headers=self.headers)

        if response.status_code == 200:
            return viewports_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def user_agents(self):
        """
        Get list of all available user agents
        :return: List of user agents (browser, device, user_agent)
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/user-agents"

        # Get response
        response = requests.get(api_url, headers=self.headers)

        if response.status_code == 200:
            return user_agents_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def async_create_scan(
        self,
        url,
        device="DESKTOP",
        user_agent=None,
        width=None,
        height=None,
        waiting_time=5,
        scanned_from="DEFAULT",
        private_scan=False,
        http_referer=None,
        submitter_tags=[],
    ):
        """
        Create async new scan, it returns an empty scan object with PENDING/RUNNING status, you have to check a few moments later if status is DONE to get full access to all scan attributes
        :param url: url to scan
        :param device: DESKTOP/MOBILE
        :param user_agent: User agents
        :param width: viewport width
        :param height: viewport height
        :param waiting_time: waiting time
        :param scanned_from: Scan country source, only for PREMIUM users
        :param private_scan: Private scan
        :param http_referer: HTTP Referer URL
        :param submitter_tags: List of user tags, only for PREMIUM users
        :return: Scan object
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/scan"

        # payload
        payload = {}
        payload["submitted_url"] = url
        if scanned_from:
            payload["scanned_from"] = scanned_from
        if device:
            payload["device"] = device
        if user_agent:
            payload["user_agent"] = user_agent
        if http_referer:
            payload["http_referer"] = http_referer
        if width:
            payload["width"] = int(width)
        if height:
            payload["height"] = int(height)
        if waiting_time:
            payload["waiting_time"] = int(waiting_time)
        if private_scan:
            payload["private_scan"] = bool(private_scan)
        if submitter_tags:
            payload["submitter_tags"] = submitter_tags

        # Get response
        response = requests.post(api_url, headers=self.headers, json=payload)

        if response.status_code == 200:
            return scan_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def create_scan(
        self,
        url,
        device="DESKTOP",
        user_agent=None,
        width=None,
        height=None,
        waiting_time=5,
        scanned_from="DEFAULT",
        private_scan=False,
        http_referer=None,
        submitter_tags=[],
    ):
        """
        Create new scan, it could be take few minutes
        :param url: url to scan
        :param device: DESKTOP/MOBILE
        :param user_agent: User agents
        :param width: viewport width
        :param height: viewport height
        :param waiting_time: waiting time
        :param scanned_from: Scan country source
        :param private_scan: Private scan
        :param http_referer: HTTP Referer URL
        :param submitter_tags: List of user tags, only for PREMIUM users
        :return: Scan object
        """
        try:
            # Create async scan
            scan = self.async_create_scan(
                url,
                device=device,
                user_agent=user_agent,
                width=width,
                height=height,
                waiting_time=waiting_time,
                scanned_from=scanned_from,
                private_scan=private_scan,
                http_referer=http_referer,
                submitter_tags=submitter_tags,
            )
            status = scan.status
            scan_result = None
            while status not in ["DONE", "ERROR"]:
                # wait 2 seconds
                time.sleep(2)
                # Get Scan
                scan_result = self.get_scan(scan.id)
                status = scan_result.scan.status
            return scan_result
        except Exception as e:
            raise ApiException(str(e))

    def fast_check(self, url):
        """
        Check if URL is CLEAN, MALICIOUS or UNRATED
        :param url: Submitted URL
        :return: FastCheck
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/fast-check"

        # payload
        payload = {"url": url}

        # Get response
        response = requests.post(api_url, headers=self.headers, json=payload)

        if response.status_code == 200:
            return fast_check_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())

    def scan_feedback(self, scan_id, feedback):
        """
        Toggle scan feedback
        :param url: url to scan
        :param feedback: MALICIOUS or SAFE
        :return: Scan Feedback object
        """
        # URL
        api_url = UrlDNA.ENDPOINT_URL + "/scan/" + str(scan_id) + "/feedback"

        # payload
        payload = {"feedback": feedback}

        # Get response
        response = requests.post(api_url, headers=self.headers, json=payload)

        if response.status_code == 200:
            return scan_feedback_schema.load(response.json())
        else:
            raise ApiException(response.content.decode())



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.scan_feedback import ScanFeedback


class ScanFeedbackSchema(Schema):
    malicious_count = fields.Integer(allow_none=True)
    safe_count = fields.Integer(allow_none=True)
    user_feedback = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return ScanFeedback(**data)


scan_feedback_schema = ScanFeedbackSchema()



class Scan(object):

    def __init__(
        self,
        id=None,
        submitted_url=None,
        scanned_from=None,
        target_url=None,
        protocol=None,
        domain=None,
        nsfw=None,
        device=None,
        user_agent=None,
        http_referer=None,
        origin=None,
        width=None,
        height=None,
        waiting_time=None,
        submitted_date=None,
        status=None,
        private_scan=None,
        submitter_tags=None,
        error_code=None,
    ):

        # Fields
        self.id = id
        self.submitted_url = submitted_url
        self.target_url = target_url
        self.protocol = protocol
        self.domain = domain
        self.nsfw = nsfw
        self.device = device
        self.user_agent = user_agent
        self.http_referer = http_referer
        self.origin = origin
        self.width = width
        self.height = height
        self.waiting_time = waiting_time
        self.submitted_date = submitted_date
        self.status = status
        self.private_scan = private_scan
        self.scanned_from = scanned_from
        self.submitter_tags = submitter_tags
        self.error_code = error_code

    def __repr__(self):
        return "<Scan id: %r, submitted_date: %r>" % (
            self.id,
            self.submitted_date.isoformat(),
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.ip_address import IpAddress


class IpAddressSchema(Schema):
    ip = fields.String(allow_none=True)
    asn = fields.String(allow_none=True)
    city = fields.String(allow_none=True)
    country = fields.String(allow_none=True)
    country_code = fields.String(allow_none=True)
    isp = fields.String(allow_none=True)
    latitude = fields.Float(allow_none=True)
    longitude = fields.Float(allow_none=True)
    org = fields.String(allow_none=True)
    region = fields.String(allow_none=True)
    timezone_gmt = fields.String(allow_none=True)
    type = fields.String(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return IpAddress(**data)


ip_address_schema = IpAddressSchema()



# Marshmallow
from marshmallow import Schema, fields, post_load

# Schemas
from urldna.schemas.scan_schema import scan_schema
from urldna.schemas.page_schema import page_schema
from urldna.schemas.cookie_schema import cookies_schema
from urldna.schemas.favicon_schema import favicon_schema
from urldna.schemas.malicious_schema import malicious_schema
from urldna.schemas.ip_address_schema import ip_address_schema
from urldna.schemas.screenshot_schema import screenshot_schema
from urldna.schemas.scan_whois_schema import scan_whois_schema
from urldna.schemas.technology_schema import technologies_schema
from urldna.schemas.certificate_schema import certificate_schema
from urldna.schemas.scan_feedback_schema import scan_feedback_schema
from urldna.schemas.redirect_chain_schema import redirect_chains_schema
from urldna.schemas.console_message_schema import console_messages_schema
from urldna.schemas.http_transaction_schema import http_transactions_schema

# Models
from urldna.models.scan_result import ScanResult


class ScanResultSchema(Schema):
    scan = fields.Nested(scan_schema, allow_none=True)
    page = fields.Nested(page_schema, allow_none=True)
    dom = fields.String(allow_none=True)
    cookies = fields.Nested(cookies_schema, allow_none=True)
    favicon = fields.Nested(favicon_schema, allow_none=True)
    malicious = fields.Nested(malicious_schema, allow_none=True)
    scan_whois = fields.Nested(scan_whois_schema, allow_none=True)
    ip_address = fields.Nested(ip_address_schema, allow_none=True)
    screenshot = fields.Nested(screenshot_schema, allow_none=True)
    certificate = fields.Nested(certificate_schema, allow_none=True)
    technologies = fields.Nested(technologies_schema, allow_none=True)
    scan_feedback = fields.Nested(scan_feedback_schema, allow_none=True)
    redirect_chains = fields.Nested(redirect_chains_schema, allow_none=True)
    console_messages = fields.Nested(console_messages_schema, allow_none=True)
    http_transactions = fields.Nested(http_transactions_schema, allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return ScanResult(**data)


scan_result_schema = ScanResultSchema()
scan_results_schema = ScanResultSchema(many=True)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt") as f:
    required = f.read().splitlines()

VERSION = "0.0.17"
DESCRIPTION = "The DNA test for websites."

setup(
    name="urlDNA",
    version=VERSION,
    author="urlDNA",
    license="MIT",
    url="https://urldna.io",
    author_email="urldna@urldna.io",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=required,
    keywords=["web scraping", "website analysis", "url scan"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# Marshmallow
from marshmallow import Schema, fields, post_load

# Model
from urldna.models.viewport import Viewport


class ViewportSchema(Schema):
    device = fields.String(allow_none=True)
    width = fields.Integer(allow_none=True)
    height = fields.Integer(allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Viewport(**data)


viewport_schema = ViewportSchema()
viewports_schema = ViewportSchema(many=True)



class Viewport(object):

    def __init__(self, device=None, width=None, height=None) -> None:
        # Fields
        self.device = device
        self.width = width
        self.height = height

    def __repr__(self):
        return "<Viewport device: %r, width: %r, height: %r>" % (
            self.device,
            self.width,
            self.height,
        )



from enum import Enum


class ScanFeedbackEnum(Enum):
    MALICIOUS = "MALICIOUS"
    SAFE = "SAFE"


class ScanFeedback(object):

    def __init__(
        self, malicious_count=None, safe_count=None, user_feedback=None
    ) -> None:
        # Fields
        self.malicious_count = malicious_count
        self.safe_count = safe_count
        self.user_feedback = user_feedback

    def __repr__(self):
        return (
            "<ScanFeedback malicious_count: %r, safe_count: %r, user_feedback: %r>"
            % (self.malicious_count, self.safe_count, self.user_feedback)
        )



# Marshmallow
from marshmallow import Schema, fields, post_load

# Models
from urldna.models.page import Page


class PageSchema(Schema):
    text = fields.String(allow_none=True)
    title = fields.String(allow_none=True)
    meta_tags = fields.Dict(keys=fields.String, values=fields.String, allow_none=True)
    headers = fields.Dict(keys=fields.String, values=fields.String, allow_none=True)
    ssdeep = fields.String(allow_none=True)
    outgoing_links = fields.List(fields.String, allow_none=True)

    @post_load
    def create(self, data, **kwargs):
        return Page(**data)


page_schema = PageSchema()



class UserAgent(object):

    def __init__(self, device=None, browser=None, user_agent=None, share=None) -> None:
        # Fields
        self.device = device
        self.browser = browser
        self.user_agent = user_agent
        self.share = share

    def __repr__(self):
        return "<UserAgent device: %r, browser: %r, user_agent: %r>" % (
            self.device,
            self.browser,
            self.user_agent,
        )



class HttpTransaction(object):

    def __init__(
        self,
        url=None,
        ip=None,
        method=None,
        mimetypes=None,
        content_type=None,
        status_code=None,
        size=None,
        phash=None,
    ) -> None:
        # Fields
        self.url = url
        self.ip = ip
        self.method = method
        self.mimetypes = mimetypes
        self.content_type = content_type
        self.status_code = status_code
        self.size = size
        self.phash = phash

    def __repr__(self):
        return "<HttpTransaction url: %r, ip: %r, content_type: %r>" % (
            self.url[:50],
            self.ip,
            self.content_type,
        )
